<?php
require_once DOL_DOCUMENT_ROOT . '/core/lib/admin.lib.php';
require_once DOL_DOCUMENT_ROOT . '/compta/facture/class/facture.class.php';
require_once DOL_DOCUMENT_ROOT . '/fourn/class/fournisseur.facture.class.php';
require_once DOL_DOCUMENT_ROOT . '/fourn/class/fournisseur.product.class.php';
require_once DOL_DOCUMENT_ROOT . '/contrat/class/contrat.class.php';


class Renew
{
    private $conf;
    private $user;
    private $db;
    var $contracts = array();
    var $invoices = array();

    function __construct($db, $conf, $user)
    {
        $this->db = $db;
        $this->conf = $conf;
        $this->user = $user;
    }

    /**
     *      Return the list of contract to renew
     *
     *      @return     array                               List of contract
     */
    public function getContracts()
    {
        return $this->contracts;
    }

    /**
     *      Add a contract to the contract list
     *
     *      @param      Contrat         $contract           Contract to add
     *      @return     int                                 < 0 on error, > 0 on success
     */
    public function addContract($contract)
    {
        if (!isset($contract))
            return -1;

        if (empty($contract->lines))
            $contract->fetch_lines();

        if (empty($contract->thirparty))
            $contract->fetch_thirdparty();

        $this->contracts[] =  $contract;
        return 1;
    }


    /**
     *      Add a contract to the contract list with its id
     *
     *      @param      int         $id                     Id of contract to add
     *      @return     int                                 < 0 on error, > 0 on success
     */
    public function addContractById($id)
    {
        $contract = new Contrat($this->db);

        if ($contract->fetch($id) <= 0)
            return -1;

        if (empty($contract->lines))
            $contract->fetch_lines();

        if (empty($contract->thirparty))
            $contract->fetch_thirdparty();

        if (empty($contract->array_options))
            $contract->fetch_optionals();

        $this->contracts[] = $contract;
        return 1;
    }

    /**
     *      Add multiple contracts to the contract list with their ids
     *
     *      @param      array         $ids                  Id of multiples contracts to add
     *      @return     int                                 < 0 on error, > 0 on success
     */
    public function addContractByIds($ids)
    {
        if (!isset($ids))
            return -1;

        $err = 0;
        foreach ($ids as $key => $id)
        {
            if ($this->addContractById($id) < 0)
                $err++;
        }
        return $err > 0 ? $err * -1 : 1;
    }

    /**
     *      Reactivate all services of a contract
     *
     *      @param      Contrat     $contract               Contract to reactivate
     *      @return     int                                 < 0 on error, number of services reactivated on success
     */
    public function reactivate($contract)
    {
        if (!isset($contract))
            return -1;

        foreach ($contract->lines as $key => $line)
        {
            if ($line->statut <= 5)
            {
                $res = $contract->close_line($this->user, $line->id, dol_now());
                if ($res < 0)
                    return $res;

                $res = $contract->active_line($this->user, $line->id, $this->conf->global->CONTRATPLUS_DATE_START,
                    $this->conf->global->CONTRATPLUS_DATE_END, $comment = $this->conf->global->CONTRATPLUS_BILL_NOTE);
                if ($res < 0)
                    return $res;

            }
        }
        return count($contract->lines);
    }

    /**
     *      Reactivate all services of each contract in $this->contracts array
     *
     *      @return     int                                 < 0 on error, > 0 on success
     */
    public function reactivateAll()
    {
        $err = 0;

        foreach ($this->contracts as $key => $contract)
        {
            if ($this->reactivate($contract) < 0)
                $err++;
        }
        return $err > 0 ? $err * -1 : 1;
    }

    /**
     *      Check if ref_supplier exist on db
     *
     *      @param      int         $ref                ref of the ref_supplier
     *      @return     bool                            true if ref exist, false if ref doesn't exist
     */
    private function refExist($ref)
    {
        $sql = 'SELECT * FROM `llx_facture_fourn` WHERE ref_supplier = ' . $ref;
        $resql = $this->db->query($sql);
        return $this->db->num_rows($resql) > 0 ? true : false;
    }

    /**
     *      Generate a reference number for an invoice (timestamp now() + id)
     *
     *      @param      int         $id                 id of the contract
     *      @return     int                             new reference number
     */
    private function generateRef($id)
    {
        return intval(dol_now()) + $id;
    }

    /**
     *      Get supplier invoice information for creation based on contract information
     *
     *      @param      Facture     $invoice            Invoice to fetch
     *      @param      Contrat     $contract           Contract used to get information
     *      @return     int                             < 0 on error, > 0 on success
     */
    private function fetchSupplierInvoiceInformation($invoice, $contract)
    {
        if (!isset($invoice, $contract))
            return -1;

        $invoice->origin_id             = $contract->id ;
        $invoice->socid				    = $contract->thirdparty->id ;
        $invoice->date           	    = $this->conf->global->CONTRATPLUS_BILL_DATE ;

        $ref_supplier = $this->generateRef($contract->id);
        while ($this->refExist($ref_supplier))
            $ref_supplier = $this->generateRef($contract->id);

        $invoice->ref_supplier		    = $ref_supplier;
        $invoice->ref_int			    = $contract->thirparty->ref_int ;
        $invoice->modelpdf			    = $contract->thirdparty->modelpdf != '' ? $contract->thirdparty->modelpdf : 'neofact';
        $invoice->cond_reglement_id	    = $contract->thirdparty->cond_reglement_id ;
        $invoice->mode_reglement_id	    = $contract->thirdparty->mode_reglement_id ;
        $invoice->remise_absolue	    = $contract->remise_absolue ;
        $invoice->remise_percent	    = $contract->remise_percent ;
        $invoice->fk_incoterms 		    = $contract->fk_incoterms ;
        $invoice->location_incoterms    = $contract->fk_incoterms ;
        $invoice->fk_account            = $contract->array_options['options_contratplus_bank'] != '' ?
            $contract->array_options['options_contratplus_bank'] :
            $this->conf->global->CONTRATPLUS_COMPTE_DEFAUT ;
        $invoice->note_public		    = isset($contract->thirdparty->note_public) ?
            $contract->thirdparty->note_public . '<br />' : '';
        $invoice->note_public           .= isset($contract->note_public) ? $contract->note_public . '<br />' : '';
        $invoice->note_public           .= isset($this->conf->global->CONTRATPLUS_BILL_NOTE) ?
            $this->conf->global->CONTRATPLUS_BILL_NOTE . '<br />' : '';
        $invoice->note_private		    = '';

        return 1;
    }

    /**
     *      Get invoice information for creation based on contract information
     *
     *      @param      Facture     $invoice            Invoice to fetch
     *      @param      Contrat     $contract           Contract used to get information
     *      @return     int                             < 0 on error, > 0 on success
     */
    private function fetchInvoiceInformation($invoice, $contract)
    {
        if (!isset($invoice, $contract))
            return -1;

        $invoice->origin_id             = $contract->id ;
        $invoice->socid				    = $contract->thirdparty->id ;
        $invoice->date           	    = $this->conf->global->CONTRATPLUS_BILL_DATE ;
        $invoice->ref_client		    = $contract->ref_customer ;
        $invoice->ref_int			    = $contract->thirparty->ref_int ;
        $invoice->modelpdf			    = $contract->thirdparty->modelpdf != '' ? $contract->thirdparty->modelpdf : 'neofact';
        $invoice->cond_reglement_id	    = $contract->thirdparty->cond_reglement_id ;
        $invoice->mode_reglement_id	    = $contract->thirdparty->mode_reglement_id ;
        $invoice->remise_absolue	    = $contract->remise_absolue ;
        $invoice->remise_percent	    = $contract->remise_percent ;
        $invoice->fk_incoterms 		    = $contract->fk_incoterms ;
        $invoice->location_incoterms    = $contract->fk_incoterms ;
        $invoice->fk_account            = $contract->array_options['options_contratplus_bank'] != '' ?
            $contract->array_options['options_contratplus_bank'] :
            $this->conf->global->CONTRATPLUS_COMPTE_DEFAUT ;
        $invoice->note_public		    = isset($contract->thirdparty->note_public) ?
            $contract->thirdparty->note_public . '<br />' : '';
        $invoice->note_public           .= isset($contract->note_public) ? $contract->note_public . '<br />' : '';
        $invoice->note_public           .= isset($this->conf->global->CONTRATPLUS_BILL_NOTE) ?
            $this->conf->global->CONTRATPLUS_BILL_NOTE . '<br />' : '';
        $invoice->note_private		    = '';

        return 1;
    }

    /**
     *      Add all services lines for the supplier invoice based on a contract
     *
     *      @param      Facture         $invoice        Invoice to fetch services
     *      @param      Contrat         $contract       Contract used to get information
     *      @return     int                             < 0 on error, > 0 on success
     */
    public function addLinesToSupplierInvoice($invoice, $contract)
    {
        $lines = $contract->lines;
        if (empty($lines) && method_exists($contract,'fetch_lines'))
        {
            $contract->fetch_lines();
            $lines = $contract->lines;
        }

        $num = count($lines);
        for ($i = 0; $i < $num; $i++)
        {
            $desc = ($lines[$i]->desc?$lines[$i]->desc:$lines[$i]->libelle);
            $product_type = ($lines[$i]->product_type?$lines[$i]->product_type:0);

            $date_start=$lines[$i]->date_debut_prevue;
            if ($lines[$i]->date_debut_reel) $date_start=$lines[$i]->date_debut_reel;
            if ($lines[$i]->date_start) $date_start=$lines[$i]->date_start;
            $date_end=$lines[$i]->date_fin_prevue;
            if ($lines[$i]->date_fin_reel) $date_end=$lines[$i]->date_fin_reel;
            if ($lines[$i]->date_end) $date_end=$lines[$i]->date_end;

            $result = $invoice->addline(
                $desc,
                $lines[$i]->subprice,
                $lines[$i]->tva_tx,
                $lines[$i]->localtax1_tx,
                $lines[$i]->localtax2_tx,
                $lines[$i]->qty,
                $lines[$i]->fk_product,
                $lines[$i]->remise_percent,
                $date_start,
                $date_end,
                0,
                $lines[$i]->info_bits,
                'HT',
                $product_type,
                $lines[$i]->rang,
                0,
                $lines[$i]->array_options,
                $lines[$i]->fk_unit
            );

            if ($result < 0)
                return -1;
        }

        $contract->fetch_lines();

        return 1;
    }

    /**
     *      Add all services lines for the invoice based on a contract
     *
     *      @param      Facture         $invoice        Invoice to fetch services
     *      @param      Contrat         $contract       Contract used to get information
     *      @return     int                             < 0 on error, > 0 on success
     */
    public function addLinesToInvoice($invoice, $contract)
    {
        if (!isset($invoice, $contract))
            return -1;

        $lines = $contract->lines;
        $fk_parent_line = 0;
        $num = count($lines);
        for ($i = 0; $i < $num; $i++)
        {
            // Don't add lines with qty 0 when coming from a shipment including all order lines
            if($contract->element == 'shipping' && $this->conf->global->SHIPMENT_GETS_ALL_ORDER_PRODUCTS && $lines[$i]->qty == 0) continue;

            $label=(! empty($lines[$i]->label)?$lines[$i]->label:'');
            $desc=(! empty($lines[$i]->desc)?$lines[$i]->desc:$lines[$i]->libelle);
            if ($invoice->situation_counter == 1) $lines[$i]->situation_percent =  0;

            if ($lines[$i]->subprice < 0)
            {
                // Negative line, we create a discount line
                $discount = new DiscountAbsolute($this->db);
                $discount->fk_soc = $invoice->socid;
                $discount->amount_ht = abs($lines[$i]->total_ht);
                $discount->amount_tva = abs($lines[$i]->total_tva);
                $discount->amount_ttc = abs($lines[$i]->total_ttc);
                $discount->tva_tx = $lines[$i]->tva_tx;
                $discount->fk_user = $this->user->id;
                $discount->description = $desc;
                $discountid = $discount->create($this->user);
                if ($discountid > 0) {
                    $invoice->insert_discount($discountid); // This include link_to_invoice
                } else {
                    setEventMessages($discount->error, $discount->errors, 'errors');
                    break;
                }
            } else {
                // Positive line
                $product_type = ($lines[$i]->product_type ? $lines[$i]->product_type : 0);

                // Reset fk_parent_line for no child products and special product
                if (($lines[$i]->product_type != 9 && empty($lines[$i]->fk_parent_line)) || $lines[$i]->product_type == 9) {
                    $fk_parent_line = 0;
                }

                // Extrafields
                if (empty($this->conf->global->MAIN_EXTRAFIELDS_DISABLED) && method_exists($lines[$i], 'fetch_optionals')) {
                    $lines[$i]->fetch_optionals($lines[$i]->rowid);
                    $array_options = $lines[$i]->array_options;
                }

                // View third's localtaxes for now
                $localtax1_tx = get_localtax($lines[$i]->tva_tx, 1, $invoice->client);
                $localtax2_tx = get_localtax($lines[$i]->tva_tx, 2, $invoice->client);

                $result = $invoice->addline($desc, $lines[$i]->subprice, $lines[$i]->qty, $lines[$i]->tva_tx, $localtax1_tx,
                    $localtax2_tx, $lines[$i]->fk_product, $lines[$i]->remise_percent, $this->conf->global->CONTRATPLUS_DATE_START,
                    $this->conf->global->CONTRATPLUS_DATE_END, 0, $lines[$i]->info_bits, $lines[$i]->fk_remise_except, 'HT', 0, $product_type,
                    $lines[$i]->rang, $lines[$i]->special_code, $invoice->origin, $lines[$i]->rowid, $fk_parent_line, $lines[$i]->fk_fournprice,
                    $lines[$i]->pa_ht, $label, $array_options, $lines[$i]->situation_percent, $lines[$i]->fk_prev_id, $lines[$i]->fk_unit);

                // Defined the new fk_parent_line
                if ($result > 0 && $lines[$i]->product_type == 9) {
                    $fk_parent_line = $result;
                }
            }
        }

        return 1;
    }

    public function createDraftFromSupplierContract($contract)
    {
        $invoice = new FactureFournisseur($this->db);

        if (!isset($contract))
            return -1;

        if ($this->fetchSupplierInvoiceInformation($invoice, $contract) < 0)
            return -2;

        $res = $invoice->create($this->user);
        if ($res < 0)
            return $res;

        if ($this->addLinesToSupplierInvoice($invoice, $contract) < 0)
            return -3;

        $res = $invoice->add_object_linked('contrat', $invoice->origin_id);
        if ($res < 0)
            return $res;

        $this->invoices[] = $invoice;
        return 1;
    }

    /**
     *      Create a draft invoice from a contract
     *
     *      @param      Contrat         $contract       Contract used to create the invoice
     *      @return     int                             < 0 on error, > 0 on success
     */
    public function createDraftFromContract($contract)
    {
        $invoice = new Facture($this->db);

        if (!isset($contract))
            return -1;

        if ($this->fetchInvoiceInformation($invoice, $contract) < 0)
            return -2;


        $res = $invoice->create($this->user, 0, getDateLim($invoice, $this->db));
        if ($res < 0)
            return $res;

        if ($this->addLinesToInvoice($invoice, $contract) < 0)
            return -3;

        $res = $invoice->add_object_linked('contrat', $invoice->origin_id);
        if ($res < 0)
            return $res;

        $this->invoices[] = $invoice;
        return 1;
    }

    /**
     *      Create draft invoice for each contract in $this->contracts array
     *
     *      @return     int                                 < 0 on error, > 0 on success
     */
    public function createDraftInvoices()
    {
        $err = 0;

        foreach ($this->contracts as $key => $contract)
        {
            if ($this->conf->global->CONTRATPLUS_SUPPLIER_CONTRACT == 1 && $contract->array_options['options_supplier_contract'] == '1') {
                if ($this->createDraftFromSupplierContract($contract) < 0)
                    $err++;
            }
            else {
                if ($this->createDraftFromContract($contract) < 0)
                    $err++;
            }
        }

        return $err > 0 ? $err * -1 : 1;
    }

    /**
     *      Generate PDF for the invoice
     *
     *      @param      CommonObject        $invoice        customer invoice or supplier invoice
     *      @return     int                                 < 0 on error, > 0 on success
     */
    private function generatePDF($invoice)
    {
        global $langs, $conf;

        $ref = dol_sanitizeFileName($invoice->ref);

        if ($invoice->element == 'invoice_supplier')
        {
            $model = isset($conf->global->CONTRATPLUS_MODEL_SUPPLIER) ?
                $this->conf->global->CONTRATPLUS_MODEL_SUPPLIER :
                $this->conf->global->INVOICE_SUPPLIER_ADDON_PDF;

        }
        else
        {
            $model = isset($conf->global->CONTRATPLUS_MODEL_CUSTOMER) ?
                $this->conf->global->CONTRATPLUS_MODEL_CUSTOMER :
                $this->conf->global->FACTURE_ADDON_PDF;
        }

        $res = $invoice->generateDocument($model, $langs);
        if ($res < 0) {
            dol_print_error($this->db, $res);
            return $res;
        }

        $res = dol_most_recent_file($this->conf->facture->dir_output . '/' . $ref, preg_quote($ref, '/').'[^\-]+');

        return $res;
    }

    /**
     *      Validate an invoice and generate a pdf for this one
     *
     *      @param      Facture         $invoice            Invoice to validate
     *      @return     int                                 < 0 on error, > 0 on success
     */
    public function validateInvoice($invoice)
    {
        if (!isset($invoice))
            return -1;

        $res = $invoice->validate($this->user);
        if ($res < 0)
            return $res;

        $invoice->fetch_lines();

        $res = $this->generatePDF($invoice);

        return $res;
    }

    /**
     *      Validate all invoice in $this->invoices array and generate a pdf for each one
     *
     *      @return     int                                 < 0 on error, > 0 on success
     */
    public function validateInvoices()
    {
        $err = 0;

        foreach ($this->invoices as $key => $invoice)
        {
            if ($this->validateInvoice($invoice) < 0)
                $err++;
        }

        return $err > 0 ? $err * -1 : 1;
    }

    /**
     *      Create a draft invoice from a $this->contracts[0]
     *
     *      @param      int          $option                Option configured by user (0, 1 or 2)
     *      @return     int                                 < 0 on error, > 0 on success
     */
    private function renewSimpleCreateInvoice($option)
    {
        if ($option >= 1)
        {
            if ($this->conf->global->CONTRATPLUS_SUPPLIER_CONTRACT == 1 && $this->contracts[0]->array_options['options_supplier_contract'] == '1') {
                if ($this->createDraftFromSupplierContract($this->contracts[0]) < 0)
                    return -1;

                if ($option == 1)
                    header("Location: " . DOL_URL_ROOT . "/fourn/facture/card.php?facid=" . $this->invoices[0]->id);
            } else {
                if ($this->createDraftFromContract($this->contracts[0]) < 0)
                    return -1;

                if ($option == 1)
                {
                    if (versioncompare(versiondolibarrarray(), array(6,0,0)) >= 0)
                        header("Location: " . DOL_URL_ROOT . "/compta/facture/card.php?facid=" . $this->invoices[0]->id);
                    else
                        header("Location: " . DOL_URL_ROOT . "/compta/facture.php?facid=" . $this->invoices[0]->id);

                }
            }
        }

        return 1;
    }

    /**
     *      Validate invoice of $this->invoices[0]
     *
     *      @param      int          $option                Option configured by user (0, 1 or 2)
     *      @return     int                                 < 0 on error, > 0 on success
     */
    private function renewSimpleValidateInvoice($option)
    {
        if ($option >= 2)
        {
            if ($this->conf->global->CONTRATPLUS_SUPPLIER_CONTRACT == 1 && $this->contracts[0]->array_options['options_supplier_contract'] == '1') {
                if ($this->validateInvoice($this->invoices[0]) < 0)
                    return -1;

                if ($option == 2)
                    header("Location: " . DOL_URL_ROOT . "/fourn/facture/card.php?facid=" . $this->invoices[0]->id);
            } else {
                if ($this->validateInvoice($this->invoices[0]) < 0)
                    return -1;

                if ($option == 2)
                {
                    if (versioncompare(versiondolibarrarray(), array(6,0,0)) >= 0)
                        header("Location: " . DOL_URL_ROOT . "/compta/facture/card.php?facid=" . $this->invoices[0]->id);
                    else
                        header("Location: " . DOL_URL_ROOT . "/compta/facture.php?facid=" . $this->invoices[0]->id);
                }
            }
        }

        return 1;
    }

    /**
     *      Renew contract of $id according to $option
     *
     *      @param      int             $option         Option configured by user (0, 1 or 2)
     *      @param      array           $id             Id of contract to renew
     *      @return     int                             < 0 on error, > 0 on success
     */
    public function renewSimple($option, $id)
    {
        if ($this->addContractById($id) < 0)
            return -1;

        if ($this->reactivate($this->contracts[0]) < 0)
            return -1;

        if ($option == 0)
            header("Location: ".$_SERVER['PHP_SELF'] . '?id=' . $this->contracts[0]->id);

        if ($this->renewSimpleCreateInvoice($option) < 0)
            return -1;

        if ($this->renewSimpleValidateInvoice($option) < 0)
            return -1;

        return 1;
    }

    /**
     *      Renew all contract of $ids according to $option
     *
     *      @param      int         $option         Option configured by user (0, 1 or 2)
     *      @param      array       $ids            Ids of contract to renew
     *      @return     int                         < 0 on error, > 0 on success
     */
    public function renewAll($option, $ids)
    {
        if ($this->addContractByIds($ids) < 0)
            return -1;

        if ($this->reactivateAll() < 0)
            return -1;

        if ($option == 0)
            header("Location: ".$_SERVER['PHP_SELF']);

        if ($this->createDraftInvoices() < 0)
            return -1;

        if ($option == 1)
            header("Location: " . DOL_URL_ROOT . "/compta/facture/list.php");

        if ($this->ValidateInvoices() < 0)
            return -1;

        if ($option == 2)
            header("Location: " . DOL_URL_ROOT . "/compta/facture/list.php");

        return 1;
    }
}